# -*- coding: utf-8 -*-
"""
Created on Mon May 11 16:40:03 2015

@author: igordsm
"""

import numpy as np
import pickle
from raw import RAWFeatureExtractor
from trios import util
from sklearn.decomposition import PCA

class PCAFeatures(RAWFeatureExtractor):
    def __init__(self, window=None, n_components=None):
        RAWFeatureExtractor.__init__(self, window)
        self.configure(window, n_components)
        
    def configure(self, window, n_components):
        self.window = window
        self.n_components = n_components
        self.temp_raw = np.zeros(np.greater(self.window, 0).sum(), np.float32)
        self.pca = PCA(self.n_components)
        
    def __len__(self):
        return self.n_components
    
    def train(self, dataset):
        X, _ = util.dataset_to_array(dataset, np.float32, False)
        self.pca.fit()        
        
    def extract(self, img, i, j, pattern):
        RAWFeatureExtractor.extract(self, img, i, j, self.temp_raw)
        pat2 = self.pca.transform(self.temp_raw)
        np.copyto(pattern, pat2)
    
    def write_state(self, obj_dict):
        RAWFeatureExtractor.write_state(self, obj_dict)
        obj_dict['n_components'] = self.n_components
        obj_dict['pca'] = pickle.dumps(self.pca)
    
    def set_state(self, obj_dict):
        RAWFeatureExtractor.set_state(self, obj_dict)
        self.configure(self.window, int(obj_dict['n_components']))
        self.pca = pickle.loads(obj_dict['pca'])
        


# -*- coding: utf-8 -*-
"""
Created on Sun Dec 13 17:35:45 2015

@author: igor
"""

import trios
from trios.feature_extractors import RAWFeatureExtractor, CombinationPattern
if trios.mp_support:
    import multiprocessing as mp
else:
    import multiprocessing.dummy as mp
import itertools

def __train_parallel(t):
    op, tr_set = t
    op.train(tr_set)
    return op


def train_combination(windows, classifiers, extractors, training1, procs=None):
    '''
    Build a combination of operators using the provided feature extractor and classifiers.
    The `procs` parameter can be used to tune the number of parallel training jobs.
    '''
    nwins = len(windows)
    ops = [trios.WOperator(windows[i], classifiers[i], 
                           extractors[i]) for i in range(nwins)]
    args = zip(ops, itertools.repeat(training1))
    pool = mp.Pool(procs)
    trained_ops = pool.map(__train_parallel, args)
    pool.close()
    pool.join()
    del pool 
    
    return CombinationPattern(*trained_ops)
    

def train_raw_combination(windows, classifiers, training1, procs=None):
    '''
    Build a combination of operators using RAW features. This function
    trains all operators using the provided windows and classifiers on 
    training1. 
    '''
    return train_combination(windows, classifiers, [RAWFeatureExtractor(w) for w in windows], training1, procs)
    


# -*- coding: utf-8 -*-
"""
Created on Wed May  6 17:00:36 2015

@author: igor
"""

import numpy as np
import sklearn
import sklearn.tree
from trios.WOperator import Classifier
from trios import util
import pickle

class DTree(Classifier):
    def __init__(self, *a, **kw):
        self.cls = sklearn.tree.DecisionTreeClassifier()
    
    def train(self, dataset, kw):
        X, y, C = util.dataset_to_array(dataset, np.float64, True, False)
        self.cls.fit(X, y, C)
    
    def apply(self, fvector):
        return self.cls.predict(fvector)[0]

    def write_state(self, obj_dict):
        obj_dict['cls'] = pickle.dumps(self.cls)
        
    def set_state(self, obj_dict):
        self.cls = pickle.loads(obj_dict['cls'])


# -*- coding: utf-8 -*-
"""
Created on Mon Apr 20 11:41:16 2015

@author: Karina
"""

import numpy as np
import sklearn
import sklearn.svm
from trios.WOperator import Classifier
from trios import util
import pickle

import theano.tensor as T
import theano

class SVM(Classifier):
    def __init__(self, *a, **kw):
        self.X = T.dmatrix('X')
        self.y = T.dvector('y')
        self.trained = False

    def train(self, dataset, **kw):
        epochs = kw['epochs'] or 10
        batch_size = kw['batch_size'] or 100
        learning_rate = kw['learning_rate'] or 0.0001
        C = 10
        Xs, ys = dataset
        ys = (ys + 1) / 2
        
        print(np.sum(ys > 0), np.sum(ys < 0), ys.shape)
        
        self.w = theano.shared(np.random.randn(Xs.shape[1]))
        self.b = theano.shared(0.0)
        decision = T.dot(self.X, self.w) + self.b
        p = T.exp(decision)
        p = p / (1 + p)
        #data_loss = T.mean(T.maximum(1 - self.y * decision, 0))
        data_loss = -T.mean(self.y*T.log(p) + (1-self.y)*T.log(1 - p))
        loss = C * data_loss + T.dot(self.w.T, self.w)
         
        self.grad_w = T.grad(loss, wrt=self.w)
        self.grad_b = T.grad(loss, wrt=self.b)
        
        self.train_theano = theano.function([self.X, self.y], [loss, data_loss], updates=[(self.w, self.w - learning_rate * self.grad_w), (self.b, self.b - learning_rate * self.grad_b)] )
        
        self.loss_theano = theano.function([self.X, self.y], data_loss)
        
        for i in range(epochs):
            print('Epoch', i)
            for k in range(0, Xs.shape[0], batch_size):
                l, h = self.train_theano(Xs[k:k+batch_size], ys[k:k+batch_size])
            print(h, self.loss_theano(Xs, ys), self.w.get_value(), self.b.get_value())

        
        predict = T.sgn(decision)
        self.predict_theano = theano.function([self.X], predict)
        self.decision_theano = theano.function([self.X], decision)
    
    def apply(self, fvector):
        res = 255 * (self.predict_theano(fvector) > 0)
        print('predict', np.sum(res > 0), np.sum(res == 0), res.shape, res)
        return res.astype('uint8')

    def write_state(self, obj_dict):
        obj_dict['svm'] = pickle.dumps(self.cls)
        
    def set_state(self, obj_dict):
        self.cls = pickle.loads(obj_dict['svm'])



# -*- coding: utf-8 -*-
"""
Created on Fri May 15 12:22:04 2015

@author: igor
"""


import numpy as np
import sklearn
from trios.WOperator import Classifier
from trios import util
import pickle

import inspect

class LinearClassifier(Classifier):
    '''
This class is a linear classifier "wrapper" that does not do training. It receives the coeficients and
bias terms in the constructor and is used mainly to use the weights computed elsewhere
inside TRIOS. 

This class can be used, for example, to use the weights computed using Linear Regression
models to do classification. 
    '''
    def __init__(self, coefs=[], bias=0,  minimize=False):
        self.coefs = coefs
        self.bias = bias
        self.minimize = minimize
    
    def train(self, dataset, kw):
        self.trained = True
    
    def apply(self, fvector):
        return 255 * (np.dot(fvector, self.coefs) + self.bias >= 0)

    def write_state(self, obj_dict):
        obj_dict['coefs'] = self.coefs
        obj_dict['bias'] = self.bias
        obj_dict['min'] = self.minimize
        
    def set_state(self, obj_dict):
        self.coefs = obj_dict['coefs']
        self.bias = obj_dict['bias']
        self.minimize = obj_dict['min']



from __future__ import unicode_literals, division

from trios.legacy._legacy import lib as v1
from trios.legacy._legacy import ffi
import trios.legacy.io as lio
from trios.WOperator import Classifier
import numpy as np

import trios.util

import math
import os
import tempfile

class ISI(Classifier):
    '''
    description here
    '''
    
    def __init__(self, win=None):
        super(ISI, self).__init__()
        self.set_win(win)
        self.trained = False
        self.ordered = False

    def set_win(self, win):
        self.win = win
        if not win is None:
            self.win_old = lio.win_to_old(win)
            self.wsize = np.sum(win != 0)
            self.wzip = math.ceil(self.wsize / 32)
        
    def train(self, dataset, kw):
        assert type(dataset) == dict
        X, y = trios.util.minimize_error(dataset)
        Xffi = ffi.cast('unsigned int *', X.ctypes.data)
        yffi = ffi.cast('unsigned int *', y.ctypes.data)
        self.itv = v1.train_operator_ISI(Xffi, yffi, self.wsize, len(dataset), self.win_old)
        self.trained = True

    def apply(self, pat):
        patffi = ffi.cast('unsigned int *', pat.ctypes.data)
        return v1.itv_list_contain(self.itv, patffi, self.itv.wzip) * 255
        
        
    def write_state(self, obj_dict):
        ftemp = tempfile.NamedTemporaryFile('w', delete=False)
        fname = bytes(ftemp.name, 'ascii')
        ftemp.close()

        v1.itv_write(fname, self.itv, self.win_old)
        with open(fname, 'r') as f:
            obj_dict['itv'] = f.read()
        os.remove(fname)
    
    def set_state(self, obj_dict):
        ftemp = tempfile.NamedTemporaryFile('w', delete=False)
        fname = bytes(ftemp.name, 'ascii')
        ftemp.write(obj_dict['itv'])
        ftemp.close()

        win_old = ffi.new('window_t **')
        self.itv = v1.itv_read(fname, win_old)
        self.set_win(lio.win_from_old(win_old[0]))
        
        os.remove(fname)



# -*- coding: utf-8 -*-
"""
Created on Fri May 15 12:22:04 2015

@author: igor
"""


import numpy as np
from trios.WOperator import Classifier
from trios import util

import inspect

class SKClassifier(Classifier):
    '''
This class wraps scikit-learn models. Any classification model should work.     
    '''
    def __init__(self, cls=None, minimize=False, ordered=True, dtype=np.uint8):
        self.cls = cls
        self.minimize = minimize
        self.ordered = ordered
        self.dtype = dtype
    
    def train(self, dataset, kw):
        if not self.ordered:
            X, y, C = util.dataset_to_array(dataset, self.dtype, True, False)            
        else:
            if len(dataset) == 2:
                X, y = dataset
                C = None
            else:
                X, y, C = dataset

        specs = inspect.getargspec(self.cls.fit)
        if C != None and 'sample_weight' in specs.args:
            self.cls.fit(X, y, C)
        else:
            if C != None:
                X2, y2 = util.expand_dataset(X, y, C)
                self.cls.fit(X2, y2)
            else:
                self.cls.fit(X, y)
    
    def apply(self, fvector):
        return self.cls.predict(fvector.reshape(1, -1))[0]

    def apply_batch(self, fmatrix):
        return self.cls.predict(fmatrix)

    def write_state(self, obj_dict):
        obj_dict['cls'] = self.cls
        obj_dict['min'] = self.minimize
        obj_dict['dtype'] = self.dtype
        obj_dict['ordered'] = self.ordered
        
    def set_state(self, obj_dict):
        self.cls = obj_dict['cls']
        self.minimize = obj_dict['min']
        self.ordered = obj_dict['ordered']
        self.dtype = obj_dict['dtype']



from ._legacy import lib as v1
from ._legacy import ffi

import numpy as np

#from trios.WOperator import WOperator
#from trios.feature_extractors import RAWBitFeatureExtractor, CombinationPattern

def win_from_old(old_win):
    h = old_win.height
    w = old_win.width
    win = np.zeros((h, w), np.uint8)
    for i in range(h):
        for j in range(w):
            if v1.win_get_point(i, j, 1, old_win) != 0: 
                win[i, j] = 1
    return win

def win_to_old(win):
    win_old = v1.win_create(win.shape[0], win.shape[1], 1)
    for i in range(win.shape[0]):
        for j in range(win.shape[1]):
            v1.win_set_point(i, j, 1, win[i, j], win_old)
    return win_old
    
def win_read(fname):
    old_win = v1.win_read(fname)
    win_np = win_from_old(old_win)
    v1.win_free(old_win)
    return win_np

'''
cpdef itv_read(char *fname):
    cdef v1.window_t *win
    cdef v1.itv_t *itv_isi
    
    itv_isi = v1.itv_read(fname, &win)
    win_np = win_from_old(win)
    #cdef ISI isi = ISI(win_np)    
    #isi.set_itv(itv_isi)
    #v1.win_free(win)
    #return WOperator(RAWBitFeatureExtractor(win_np), isi)

cpdef operator_read(fname):
    # add error handling in case of two level operators?
    fname = fname + '-files/operator'
    return itv_read(fname)

cpdef two_level_operator_read(fname):
    with open(fname) as f:
        lines = f.readlines()
        n_ops = int(lines[2].split()[0])
    ops = []
    for i in range(n_ops):
        opfname = '%s-files/level0/operator%d'
        ops.append(operator_read(opfname))
    comb = CombinationPattern(*ops, bitpack=True)
    op2nd = operator_read('%s-files/level1/operator0')
    wop = WOperator(comb, op2nd.classifier)
    
'''


"""
Contains Imageset related functions.
"""

import os.path
import numpy as np

class Imageset(list):
    """
An Imageset is a list of examples (tuples containing two or three strings). Each example contains the path of the input and ideal images and may contain a binary mask. Imageset is a subclass of list.

Every place that expects an Imageset will work with a simple list in the following format:

[ 
 ['input1', 'ideal1', 'mask1'],
 ['input2', 'ideal2'],
 ....
 ....
]

Also, Imageset([['input1', 'ideal1', 'mask1'], ['input2', 'ideal2'], ... ]) converts the list to an Imageset.
    """
    
    @staticmethod
    def __skip_blank(lines, count):
        while lines[count].strip() == "":
            count += 1
        return count
    
    @classmethod
    def read(cls, fname):
        """
        Reads Imageset from disk.
        """
        f = open(fname, 'r')
        imgset = Imageset()
        
        lines = f.readlines()
        
        n = int(lines[1].split()[1])
        ngroups = int(lines[2].split()[1])
        if ngroups != 2 and ngroups != 3:
            print('Number of groups must be 2 or 3')
            return None
        
        base_dir = [""] * 3
        base_dir[0] = lines[4].strip()
        base_dir[1] = lines[5].strip()
        
        pref = os.path.dirname(fname)
        
        if ngroups == 3:
            base_dir[2] = lines[6].strip()
            
        for i in range(3):
            if (base_dir[i].startswith('./') and 
                len(base_dir[i].split('/')) == 2):
                
                base_dir[i] = base_dir[i][2:]
            
        for i in range(3):
            base_dir[i] = os.path.join(pref, base_dir[i])
            
        line_count = 7
        for i in range(n):
            line_count = cls.__skip_blank(lines, line_count)
            input_img = base_dir[0] + lines[line_count].strip()
            ideal_img = base_dir[1] + lines[line_count+1].strip()
            if ngroups == 2:
                imgset.append( (input_img, ideal_img, None) )
            else:
                mask_img = base_dir[2] + lines[line_count+2].strip()
                imgset.append( (input_img, ideal_img, mask_img) )
            line_count += ngroups        
        
        f.close()
        return imgset
        
    def append(self, example):
        """
        Adds an example to the Imageset. All examples must have the same number of images.
        """
        if len(example) != 2 and len(example) != 3:
            return 
    
        if len(self) == 0:
            super(Imageset, self).append(example)
        elif len(example) == len(self[0]):
            super(Imageset, self).append(example) 
        
    def write(self, fname):
        """
        Writes Imageset to disk.
        """
        f = open(fname, 'w')
        f.write('IMGSET  ########################################################\n')
        
        n = len(self)
        ngroups = len(self[0])
        
        f.write('.n %d\n' % n)
        f.write('.f %d\n' % ngroups)
        f.write('.d\n' + './\n' * ngroups + '\n')
        
        for example in self:
            f.write('\n'.join(example))
            f.write('\n\n')
        
        f.close()
        #Show?
    



import numpy as np

def square(sz):
    return np.ones((sz, sz), np.uint8)

def rectangle(h, w):
    return np.ones((h, w), np.uint8)
    

def random_win(domain_window, sz, include_center=False, rndstate=np.random):
    y, x = np.nonzero(domain_window)
    perm = rndstate.permutation(len(x))[:sz]
    win = np.zeros(domain_window.shape, np.uint8)
    win[y[perm], x[perm]] = 1
    if include_center:   
        win[domain_window.shape[0]//2, domain_window.shape[1]//2] = 1
    return win

def show(win, fname=None):
    import matplotlib # matplotlit is an optional dependency
    import matplotlib.pyplot as plt
    
    fig = plt.figure()
    ax = fig.add_subplot(111)
    h, w = win.shape
    plt.xlim(0, w)
    plt.ylim(0, h)
    
    for i in range(h):
        for j in range(w):
            if win[i, j]:
                p = matplotlib.patches.Circle( (j + 0.5, h - i - 0.5), 0.35, color='black')
                ax.add_patch(p)
    
    for i in range(h):
        l = matplotlib.lines.Line2D([0, w], [i, i], color='black')
        ax.add_line(l)
        
    for j in range(w):
        l = matplotlib.lines.Line2D([j, j], [0, h], color='black')
        ax.add_line(l)
    if fname == None:
        plt.show()
    else:
        plt.savefig(fname)




# -*- coding: utf-8 -*-
"""
Created on Mon Apr 13 13:49:26 2015

@author: igordsm
"""

from __future__ import print_function

import theano.tensor as T
import theano
import math
import numpy as np
import sys

from trios.WOperator import FeatureExtractor
from trios.feature_extractors import RAWFeatureExtractor

from trios import util

from StringIO import StringIO

class AutoEncoder(FeatureExtractor):
    def __init__(self, window=None, n_hidden=100, hidden_nl='Logit'):
        '''
            Calculates the representation
                y = s(Wx + b1)
            The original data can be retrieved using
                z = s(W'y + b2)
        '''
        
        FeatureExtractor.__init__(self)
        if not window is None:
            self.configure(window, n_hidden, hidden_nl)
        
    def configure(self, window, n_hidden, hidden_nl):
        self.window = window
        self.n_inputs = np.greater(self.window, 0).sum()
        self.n_hidden = n_hidden
        
        self.raw = RAWFeatureExtractor(window, 1.0/255)
        self.raw_pat = np.zeros(self.n_inputs, theano.config.floatX)
        
        initial_W = np.asarray(
                np.random.uniform(
                    low=-4 * np.sqrt(6. / (self.n_inputs + self.n_hidden)),
                    high=4 * np.sqrt(6. / (self.n_inputs + self.n_hidden)),
                    size=(self.n_inputs, self.n_hidden)
                ),
                dtype=theano.config.floatX)
             
        self.W = theano.shared(initial_W, name='W')
        self.X = T.matrix('x') # this is the input
        
        self.b1 = theano.shared(np.zeros(self.n_hidden, theano.config.floatX), 'b1')
        
        self.hidden_nl = hidden_nl
        if hidden_nl == 'Logit':
            self.y = T.nnet.sigmoid(T.dot(self.X, self.W) + self.b1)
        else:
            self.y0 = T.dot(self.X, self.W) + self.b1
            self.y = T.switch(self.y0 > 0, self.y0, 0)

        self.b2 = theano.shared(np.zeros(self.n_inputs, theano.config.floatX), 'b2')
        self.z = T.nnet.sigmoid(T.dot(self.y, self.W.T) + self.b2)
        self.z_round = T.gt(self.z, 0.5)
        self.params = [self.W, self.b1, self.b2]
        
        
        self.cost_mse = T.sum( (self.z - self.X)*(self.z - self.X), axis=1).mean()
        self.cost_ce = -T.sum( self.X * T.log(self.z) +  (1 - self.X) * T.log(1 - self.z), axis=1  ).mean()
        self.cost_class = T.sum(T.abs_(self.z_round - self.X), axis=1).mean()
        
        self.cost_dict = {'MSE': self.cost_mse, 'CE': self.cost_ce, 'CLASS': self.cost_class}
        
        self.__encode = theano.function(inputs=[self.X], outputs=[self.y])
        
    
    def write_state(self, obj_dict):
        super(AutoEncoder, self).write_state(obj_dict)
        obj_dict['n_hidden'] = self.n_hidden
        obj_dict['hidden_nl'] = self.hidden_nl
        
        f = StringIO()
        np.savetxt(f, self.W.get_value())
        obj_dict['W'] = f.getvalue()
        f.close()

        f = StringIO()
        np.savetxt(f, self.b1.get_value())
        obj_dict['b1'] = f.getvalue()
        f.close()

        f = StringIO()
        np.savetxt(f, self.b2.get_value())
        obj_dict['b2'] = f.getvalue()
        f.close()

    def set_state(self, obj_dict):
        # parameters: window, n_hidden, hidden_nl, W, b1, b2
        super(AutoEncoder, self).set_state(obj_dict)
        n_hidden = int(obj_dict['n_hidden'])
        self.configure(self.window, n_hidden, obj_dict['hidden_nl'])
        
        f = StringIO(obj_dict['W'])
        self.W.set_value(np.loadtxt(f, theano.config.floatX))

        f = StringIO(obj_dict['b1'])
        self.b1.set_value(np.loadtxt(f, theano.config.floatX))
        
        f = StringIO(obj_dict['b2'])
        self.b2.set_value(np.loadtxt(f, theano.config.floatX))
                
        
    def __len__(self):
        self.n_hidden
    
    def temp_feature_vector(self):
        return np.zeros(self.n_hidden, theano.config.floatX)
    
    def train(self, dataset, epochs=1, learning_rate=0.01, batch_size=100, error='MSE', l2_coef=0, l1_coef=0):
        '''
            Takes a dataset and builds a matrix with n_inputs columns and len(dataset)
            rows apply SGD with a given number of epochs.
        '''
        if not error in self.cost_dict:
            raise ValueError("Error function not found: %s"%error)
        cost_fn = self.cost_dict[error] + l2_coef * T.sum(self.W ** 2) + l1_coef * T.sum(abs(self.W))

        if isinstance(dataset, np.ndarray):
            X = dataset
        else:
            X, _ = util.dataset_to_array(dataset, theano.config.floatX, False)
        m = X.shape[0]
        
        # define the updates  'functions', involves calculating cost function and grad(cost, params)
        grads = T.grad(cost_fn, self.params)
        updates = []
        for p, g in zip(self.params, grads):
            updates.append( (p,  p - learning_rate * g ) )
        
        # define train theano function
        sgd = theano.function(inputs=[self.X], outputs=[cost_fn], updates=updates)
        for e in range(epochs):
            nbatches = int(math.ceil(m / batch_size))            
            print('Epoch', e, file=sys.stderr)
            for b in range(nbatches):
                cost = sgd(X[b*batch_size:(b+1)*batch_size])[0]
                #print('%d - %f'%(b, cost))
                
    def reconstruction_error(self, dataset, error='MSE'):
        if not error in self.cost_dict:
            raise ValueError("Error function not found: %s"%error)
        cost_fn = self.cost_dict[error]
            
        if isinstance(dataset, np.ndarray):
            X = dataset
        else:
            X, _ = util.dataset_to_array(dataset, theano.config.floatX, False)
        
        evl = theano.function(inputs=[self.X], outputs=[cost_fn])
        return evl(X)[0]  
    
    def encode(self, pat):
        return self.__encode(pat)[0]

    def extract(self, img, i, j, pat):
        '''
            Apply AE and return the encoded input
        '''
        self.raw.extract(img, i, j, self.raw_pat)
        encoded = self.encode(self.raw_pat.reshape(1, self.n_inputs))
        np.copyto(pat, encoded)
        


# -*- coding: utf-8 -*-
"""
Created on Fri Dec 11 10:11:40 2015

@author: igor
"""
from __future__ import print_function

import trios
import numpy as np
import scipy as sp
import scipy.ndimage
import os
import os.path

from trios.wop_matrix_ops import compare_images, compare_images_binary

import sys

def __apply_parallel(op, elem):
    pass

def apply_and_save(operator, testset, folder_prefix, procs=None):
    '''
    Applies an operator in all images of a given testset. The
    images are saved in folder_prefix with the same name as the
    groundtruth images.
    '''
    
    pass

def compare_folders(testset, res_folder, binary=True):
    '''
    Computes the accuracy of a set of processed images saved
    on the disk. Takes as input a testset and the folder where the
    result images are stored. Returns
    the accuracy.
    '''
    
    if binary:
        perf = np.zeros(5, np.uint32)
    else:
        perf = np.zeros(2, np.uint32)
    for (i, o, m) in testset:
        print('Comparing', o, file=sys.stderr)
        out = sp.ndimage.imread(o, mode='L')
        o_name = os.path.split(o)[1]
        msk = sp.ndimage.imread(m, mode='L')
        res = sp.ndimage.imread('%s/%s'%(res_folder, o_name), mode='L')
        
        if binary:
            perf_i = compare_images_binary(out, msk, res, 0, 0)
        else:
            perf_i = compare_images(out, msk, res, 0, 0)
        perf = perf + np.asarray(perf_i)
    
    if binary:
        return perf[:4]
    else:
        return perf[0] / perf[1]
    
    



from setuptools.extension import Extension
from setuptools import setup
import os
import platform

try:
    import Cython
    use_cython = True
except ImportError:
    use_cython = False

if os.path.exists('trios/WOperator.pyx'):
    ext = '.pyx'
else:
    ext = '.c'

import numpy as np

includes = [np.get_include()]#, 'C:/Users/Igor/Miniconda3/includes']
if platform.system() == 'Linux':
    extra_compile_args = ['-fopenmp']
    extra_link_args = ['-fopenmp']

extensions = [
            Extension('trios.WOperator', [
                    'trios/WOperator'+ext]),
            Extension('trios.feature_extractors.raw', [
                    'trios/feature_extractors/raw'+ext]),
            Extension('trios.feature_extractors.aperture', [
                    'trios/feature_extractors/aperture'+ext]),        
            Extension('trios.feature_extractors.combination', [
                    'trios/feature_extractors/combination'+ext]),            
            Extension('trios.wop_matrix_ops', [
                    'trios/wop_matrix_ops'+ext]),
            Extension('trios.util', [
                    'trios/util'+ext]),
            Extension('trios.serializable', [
                    'trios/serializable'+ext]),
            Extension('trios.window_determination.relief', [
                    'trios/window_determination/relief'+ext
                    ])]

for extt in extensions:
    extt.include_dirs += includes
    if platform.system() == 'Linux':
        extt.extra_compile_args += extra_compile_args
        extt.extra_link_args += extra_link_args

if use_cython:
    from Cython.Build import cythonize
    extensions = cythonize(extensions)

setup(
    name='trios',
    version='2.0.8',
    author='Igor Montagner, Roberto Hirata Jr, Nina S. T. Hirata',
    author_email='igordsm+trios@gmail.com',
    url='http://trioslib.sf.net',
    classifiers=['Development Status :: 4 - Beta',
                 'Programming Language :: Python :: 2',
                 'Programming Language :: Python :: 3'],
    install_requires=[
        'cython', 'cffi', 'numpy', 'scikit-learn', 'scipy', 'pillow'
    ], 
    ext_modules = extensions,
    cffi_modules = ['trios/legacy/build_legacy.py:c_code'],
    packages = ['trios', 'trios.feature_extractors', 'trios.classifiers', 'trios.legacy', 'trios.window_determination', 'trios.shortcuts',]# 'trios.contrib']
)





import pickle
import gzip
import scipy as sp
import scipy.ndimage

def save_gzip(op, fname):
    with gzip.open(fname, 'wb') as f:
        pickle.dump(op, f, -1)

def load_gzip(fname):
    with gzip.open(fname, 'rb') as f:
        return pickle.load(f)

def load_image(fname):
    return sp.ndimage.imread(fname, mode='L')

def save_image(image, fname):
    sp.misc.imsave(fname, image)


from cffi import FFI

c_code = FFI()

sources = ['trios/legacy/' + f for f in ['basic_win.c', 'basic_error.c', 'io_header.c', 'io_win.c', 'basic_itv.c', 'io_itv.c', 'basic_mtm.c', 'io_mtm.c', 'basic_common.c']]

sources.extend(['trios/legacy/ISI/' + f for f in ['dp_isi.c', 'ip_isi.c', 'ip_mincover.c', 'partition.c', 'train_operator.c']])


c_code.cdef('''
typedef struct {
  unsigned char width ;  /*!< window matrix width (1<=width<=256)  */
  unsigned char height ; /*!< window matrix height (1<=width<=256) */
  unsigned char nbands ; /*!< window matrix bands (1<=bands<=256)  */
  int  *wsize ;          /*!< window size                          */
  char *windata ;        /*!< window description array             */
} window_t ;

window_t *win_read(char *fname);
window_t *win_create(int height, int width, int nbands);
void win_free(window_t *);
int win_set_point(int, int, int, int, window_t *);
int win_get_point(int, int, int, window_t *);


typedef struct itv_bx { /*!< Structure of the interval                      +*/
  unsigned int *A ;     /*!< left extremity of the interval                 +*/
  unsigned int *B ;     /*!< right extremity of the interval                +*/
  int label ;           /*!< value associated to the points of the interval +*/
  struct itv_bx *next ; /*!< pointer to the next interval                   +*/
} itv_BX ;

typedef struct {
  int  wsize ;          /*!< w-pattern size +*/
  int  wzip ;           /*!< compacted w-pattern size +*/
  int  type ;           /*!< type (BB, BG, GB, GG) +*/
  int  nitv ;           /*!< number of intervals +*/
  int  maxlabel ;       /*!< maximum label assigned to the intervals in the set +*/
  int  deflabel ;       /*!< default value to be associated to the
                            points out of the intervals union +*/
  int  *head ;          /*!< pointer to the intervals list head +*/
} itv_t ;

itv_t *itv_read(char *fname, window_t ** win);
int itv_write(char *fname, itv_t * itv, window_t * win);

/* add stuff for isi here */
itv_t *train_operator_ISI(unsigned int *data, unsigned int *freqs, int wsize, int npat, window_t *win);
int itv_contain(itv_BX *p_itv, unsigned int *wpat, int wzip);
int itv_list_contain(itv_t *itv, unsigned int *wpat, int wzip);
''')


c_code.set_source('trios.legacy._legacy', '''
#include "trios_win.h"
#include "trios_itv.h"
#include "trios_error.h"
#include "trios_mtm.h"
#include "trios_misc.h"

#include "trios_learn.h"
#include "paclearn_local.h"

itv_t *train_operator_ISI(unsigned int *data, unsigned int *freqs, int wsize, int npat, window_t *win);


''', sources=sources, include_dirs=['trios/legacy/', 'trios/legacy/ISI/'])

if __name__ == '__main__':
    c_code.compile()
